/*
    WaveInfoCmd.cc

    Will EIghteen: will@opaline.demon.co.uk
*/

#include "WaveInfoCmd.h"
#include "MixwaveUtils.h"
#include <DMenuCmd.h>
#include "OpResources.h"
#include "AppWindow.h"	// access to wave loaded at top level
#include <InfoDialogManager.h>
#include <MessageDialogManager.h>


WaveInfoCmd::WaveInfoCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_menuRealised = False;
}


void WaveInfoCmd::doCommand()
{
	unsigned int n;
	unsigned int maxNameLength = 0;
	
	if ( (*wavePak)[0].pWave->first() == NULL )
		return;
		
	if ( _menuRealised == True )
		return;

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( ( char * ) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Quit" );

	_dMenu = new DMenuCmd ( "Wave info menu", TRUE, FALSE,
					&WaveInfoCmd::dMenuSelectionCallback,
					( void * ) this, "Current wave files",
					_arrayItems, maxNameLength, ( char * ) &strings );

	_menuRealised = True;
	_dMenu->execute();
}


void WaveInfoCmd::dMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	WaveInfoCmd *obj = ( WaveInfoCmd * ) clientData;				

	// Since the menu is static with reference to the name strings
	// (quitOnSelect is false), the reference may be invalid; check by name.

	if ( strcmp ( label, "Quit" ) == 0 )
	{
		obj->_menuRealised = False;			// menu will be popped down
		delete obj->_dMenu;
	}
	else
	{
		int count, found;

		// scan base list for returned name
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			found = 0;
			
			if ( strcmp ( label, (*wavePak)[count].pWave->name() ) == 0 )
			{
				obj->waveInfo ( (*wavePak)[count].pWave->waveNum() );
				found = 1;
				count = wavePak->Count();
			}
		}
		
		if ( !found && mixwaveResources->enableMessagePopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}
}


void WaveInfoCmd::waveInfo ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )
	{
		_menuRealised = False;
		return;
	}

	char buf[4096];
	char intText[20];

	MixwaveUtils *utils = new MixwaveUtils;

	for ( int count = 0; count < wavePak->Count(); count++ )
		if ( (*wavePak)[count].infoNum == selectedItemNum )
			return;	// info is already displayed for this wave

	strncpy ( buf, (*wavePak)[selectedItemNum-1].pWave->name(), sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, " ", sizeof ( buf ) - strlen ( buf ) - 1 );
     
	if ((*wavePak)[selectedItemNum-1].pWave->totalSamples() == 0) 
		strncat ( buf, " initialised. \n", sizeof ( buf ) - strlen ( buf ) - 1 );
	else
	{
  		strncat ( buf, "\nFile type ", sizeof ( buf ) - strlen ( buf ) - 1 );
		snprintf ( intText, sizeof ( intText ), "%d", (*wavePak)[selectedItemNum-1].pWave->waveType() );
		intText [ sizeof ( intText ) - 1 ] = 0;
		strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );

		switch((*wavePak)[selectedItemNum-1].pWave->waveType())
		{
			case 1:   	
				strncat  ( buf, ": linear\n", sizeof ( buf ) - strlen ( buf ) - 1 );
			 	strncat  ( buf, "Channels: ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%d\n", (*wavePak)[selectedItemNum-1].pWave->numChannels() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat  ( buf, "Bits/sample: ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%d\n", (*wavePak)[selectedItemNum-1].pWave->bitsPerSample() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat  ( buf, "Sample rate: ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%ld/sec\n", (*wavePak)[selectedItemNum-1].pWave->sampleRate() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat  ( buf, "Data size: ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%ld", (*wavePak)[selectedItemNum-1].pWave->dataLength() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat  ( buf, " bytes\n", sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat  ( buf, "Samples: ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%ld (", (*wavePak)[selectedItemNum-1].pWave->totalSamples() );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncpy ( intText, utils->sampleNumToTimeString (
					(*wavePak)[selectedItemNum-1].pWave->totalSamples(),
					(*wavePak)[selectedItemNum-1].pWave->sampleRate(),
					sizeof ( intText ), mixwaveResources->fpDigits() ), sizeof ( intText ) );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat ( buf, " mins:secs)\n", sizeof ( buf ) - strlen ( buf ) - 1 );

				if ( (*wavePak)[selectedItemNum-1].graphNum == 0 )	// or the wavePak selection isn't the current one in the viewer
				{
					if ( (*wavePak)[selectedItemNum-1].startSel != (*wavePak)[selectedItemNum-1].endSel )
					{
						strncat ( buf, "Selection on last view popdown:\n", sizeof ( buf ) - strlen ( buf ) - 1 );
						snprintf ( intText, sizeof ( intText ), "%ld", (*wavePak)[selectedItemNum-1].startSel );
						intText [ sizeof ( intText ) - 1 ] = 0;
						strncat ( buf, intText,sizeof ( buf ) - strlen ( buf ) - 1 );
						strncat ( buf, " to ", sizeof ( buf ) - strlen ( buf ) - 1 );
						snprintf ( intText, sizeof ( intText ), "%ld samples ", (*wavePak)[selectedItemNum-1].endSel );
						intText [ sizeof ( intText ) - 1 ] = 0;
						strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
					}

					if ( (*wavePak)[selectedItemNum-1].startRange != (*wavePak)[selectedItemNum-1].endRange )
					{
						snprintf ( intText, sizeof ( intText ), "(%ld loaded)\n", (*wavePak)[selectedItemNum-1].intArraySize );
						intText [ sizeof ( intText ) - 1 ] = 0;
						strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
					}
				}

				break;	// end case 1: PCM			
        }	// end switch waveType
     }

	delete utils;
	(*wavePak)[selectedItemNum-1].infoNum = (*wavePak)[selectedItemNum-1].pWave->waveNum();

	infoDialogManager->post ( buf, ( void * ) (*wavePak)[selectedItemNum-1].pWave,
								&WaveInfoCmd::infoOkCallback,
								&WaveInfoCmd::infoCancelCallback );
}


void WaveInfoCmd::infoOkCallback ( void *clientData )
{
	Wave *pWave = ( Wave * ) clientData;
	
	(*wavePak)[pWave->waveNum()-1].infoNum = 0;	// re-enable info popup
}


void WaveInfoCmd::infoCancelCallback ( void *clientData )
{
	Wave *pWave = ( Wave * ) clientData;

	(*wavePak)[pWave->waveNum()-1].infoNum = 0;	// re-enable info popup
}
